var fs = require('fs');
var path = require('path');
var babel = require('babel');

var bReactContent = fs.readFileSync(path.join(__dirname, 'bReact.js'), 'utf8');
var bReactASTBody = babel.parse(bReactContent).body[0];

function getOptions(file) {
  var opts = file.opts.extra['react-bemed'] || {};

  return {
    elemDelim: opts.elemDelim || '__',
    modDelim: opts.modDelim || '_',
    bevis: opts.bevis || false
  };
}

function camelToDash(input) {
  return input.replace(/([A-Z])/g, function (i) {
    return '-' + i.toLowerCase();
  }).replace(/^-/, '');
}

function safeGetter(obj, getPath) {
  getPath = getPath.split('.');

  var result = obj;
  for (var i = 0; i < getPath.length; i++) {
    result = result[getPath[i]];
    if (!result) {
      return null;
    }
  }

  return result;
}

module.exports = function (_ref) {
  var Plugin = _ref.Plugin;
  var t = _ref.types;

  return new Plugin('plugin-smacker', {
    visitor: {
      Program: function Program(node, parent, scope, file) {
        var opts = getOptions(file);

        // set options from settings
        var properties = bReactASTBody.declarations[0].init.properties;
        properties[0].value = t.literal(opts.modDelim);
        properties[1].value = t.literal(opts.bevis);
        bReactASTBody._compact = true;

        this.unshiftContainer('body', bReactASTBody);
      },

      JSXOpeningElement: function JSXOpeningElement(node, parent, scope, file) {
        if (!node.attributes.length) {
          return;
        }

        var opts = getOptions(file);

        var blockName = scope.data.__bemBlockName;

        if (!blockName) {
          // A functional component using an ES2015 (ES6) arrow function
          if (safeGetter(scope, 'block.type') === 'ArrowFunctionExpression' && safeGetter(scope, 'parentBlock.type') === 'VariableDeclarator') {
            blockName = scope.parentBlock.id.name;
            // Class component
          } else if (safeGetter(scope, 'parent.block.type') === 'ClassDeclaration') {
              blockName = scope.parent.block.id.name;
            }

          if (blockName) {
            blockName = camelToDash(blockName);
            scope.data.__bemBlockName = blockName;
          } else {
            // Don't process unknown constractions
            return;
          }
        }

        node.attributes.forEach(function (attr) {
          if (attr.name.name === 'bElem') {
            attr.value = t.literal(blockName + opts.elemDelim + attr.value.value);
            return;
          }

          if (attr.name.name === 'bBlock') {
            attr.value = t.literal(blockName);
          }
        });
      }
    }
  });
};